"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exploreCertificateSigningRequest = exports.readCertificationRequestInfo = void 0;
var asn1_js_1 = require("./asn1.js");
var crypto_explore_certificate_js_1 = require("./crypto_explore_certificate.js");
function _readExtensionRequest(buffer) {
    var block = (0, asn1_js_1.readTag)(buffer, 0);
    var inner_blocks = (0, asn1_js_1._readStruct)(buffer, block);
    var extensions = inner_blocks.map(function (block1) { return (0, crypto_explore_certificate_js_1._readExtension)(buffer, block1); });
    var result = {};
    for (var _i = 0, extensions_1 = extensions; _i < extensions_1.length; _i++) {
        var e = extensions_1[_i];
        result[e.identifier.name] = e.value;
    }
    var basicConstraints = result.basicConstraints, keyUsage = result.keyUsage, subjectAltName = result.subjectAltName;
    return { basicConstraints: basicConstraints, keyUsage: keyUsage, subjectAltName: subjectAltName };
}
function readCertificationRequestInfo(buffer, block) {
    var blocks = (0, asn1_js_1._readStruct)(buffer, block);
    if (blocks.length === 4) {
        var extensionRequestBlock = (0, asn1_js_1._findBlockAtIndex)(blocks, 0);
        if (!extensionRequestBlock) {
            throw new Error("cannot find extensionRequest block");
        }
        var blocks1 = (0, asn1_js_1._readStruct)(buffer, extensionRequestBlock);
        var blocks2 = (0, asn1_js_1._readStruct)(buffer, blocks1[0]);
        var identifier = (0, asn1_js_1._readObjectIdentifier)(buffer, blocks2[0]);
        if (identifier.name !== "extensionRequest") {
            throw new Error(" Cannot find extension Request in ASN1 block");
        }
        var buf = (0, asn1_js_1._getBlock)(buffer, blocks2[1]);
        var extensionRequest = _readExtensionRequest(buf);
        return { extensionRequest: extensionRequest };
    }
    throw new Error("Invalid CSR or ");
}
exports.readCertificationRequestInfo = readCertificationRequestInfo;
function exploreCertificateSigningRequest(crl) {
    var blockInfo = (0, asn1_js_1.readTag)(crl, 0);
    var blocks = (0, asn1_js_1._readStruct)(crl, blockInfo);
    var csrInfo = readCertificationRequestInfo(crl, blocks[0]);
    return csrInfo;
}
exports.exploreCertificateSigningRequest = exploreCertificateSigningRequest;
//# sourceMappingURL=explore_certificate_signing_request.js.map