"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAttributes = void 0;
var common_js_1 = require("../common.js");
var _crypto_js_1 = require("./_crypto.js");
var keyUsageApplication = _crypto_js_1.x509.KeyUsageFlags.keyEncipherment |
    _crypto_js_1.x509.KeyUsageFlags.nonRepudiation |
    _crypto_js_1.x509.KeyUsageFlags.dataEncipherment |
    _crypto_js_1.x509.KeyUsageFlags.keyCertSign |
    _crypto_js_1.x509.KeyUsageFlags.digitalSignature;
var keyUsageCA = _crypto_js_1.x509.KeyUsageFlags.keyCertSign | _crypto_js_1.x509.KeyUsageFlags.cRLSign;
function getAttributes(purpose) {
    var basicConstraints;
    var keyUsageExtension = [];
    var usages;
    var nsComment;
    var extension;
    switch (purpose) {
        case common_js_1.CertificatePurpose.ForCertificateAuthority:
            extension = "v3_ca";
            basicConstraints = new _crypto_js_1.x509.BasicConstraintsExtension(true, undefined, false);
            usages = keyUsageCA;
            keyUsageExtension = [];
            nsComment = "Self-signed certificate for CA generated by Node-OPCUA Certificate utility V2";
            break;
        case common_js_1.CertificatePurpose.ForApplication:
        case common_js_1.CertificatePurpose.ForUserAuthentication:
        default:
            extension = "v3_selfsigned";
            basicConstraints = new _crypto_js_1.x509.BasicConstraintsExtension(false, undefined, true);
            usages = keyUsageApplication;
            keyUsageExtension = [_crypto_js_1.x509.ExtendedKeyUsage.serverAuth, _crypto_js_1.x509.ExtendedKeyUsage.clientAuth];
            nsComment = "Self-signed certificate generated by Node-OPCUA Certificate utility V2";
            break;
    }
    return { nsComment: nsComment, basicConstraints: basicConstraints, keyUsageExtension: keyUsageExtension, usages: usages };
}
exports.getAttributes = getAttributes;
//# sourceMappingURL=_get_attributes.js.map