"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.performMessageChunkManagerTest = exports.verifyMessageChunkSignatureForTest = exports.construct_verifyMessageChunkSignatureForTest = exports.makeMessageChunkSignatureForTest = void 0;
const fs = __importStar(require("fs"));
const node_crypto_1 = require("node:crypto");
require("should");
const node_opcua_assert_1 = require("node-opcua-assert");
const node_opcua_crypto_1 = require("node-opcua-crypto");
const node_opcua_service_secure_channel_1 = require("node-opcua-service-secure-channel");
const source_1 = require("../source");
// tslint:disable:no-var-requires
const { getFixture } = require("node-opcua-test-fixtures");
function construct_makeMessageChunkSignatureForTest() {
    const privateKey = (0, node_crypto_1.createPrivateKey)(fs.readFileSync(getFixture("certs/server_key_1024.pem"), "utf-8"));
    return (chunk) => {
        const options = {
            algorithm: "RSA-SHA256",
            privateKey,
            signatureLength: 128,
        };
        const buf = (0, node_opcua_crypto_1.makeMessageChunkSignature)(chunk, options); // Buffer
        (0, node_opcua_assert_1.assert)(buf instanceof Buffer, "expecting a Buffer");
        return buf;
    };
}
exports.makeMessageChunkSignatureForTest = construct_makeMessageChunkSignatureForTest();
function construct_verifyMessageChunkSignatureForTest() {
    const publicKey = fs.readFileSync(getFixture("certs/server_public_key_1024.pub")).toString("utf-8");
    return (chunk) => {
        (0, node_opcua_assert_1.assert)(chunk instanceof Buffer);
        const options = {
            algorithm: "RSA-SHA256",
            publicKey,
            signatureLength: 128
        };
        return (0, node_opcua_crypto_1.verifyChunkSignature)(chunk, options);
    };
}
exports.construct_verifyMessageChunkSignatureForTest = construct_verifyMessageChunkSignatureForTest;
exports.verifyMessageChunkSignatureForTest = construct_verifyMessageChunkSignatureForTest();
function performMessageChunkManagerTest(options) {
    const securityHeader = new node_opcua_service_secure_channel_1.SymmetricAlgorithmSecurityHeader();
    const bodySize = 32;
    const headerSize = 12 + securityHeader.binaryStoreSize();
    options.signatureLength = options.signatureLength || 0; // 128 bytes for signature
    options.chunkSize = bodySize + options.signatureLength + headerSize + 8; // bodySize useful bytes
    options.requestId = 1;
    const sequenceNumberGenerator = new source_1.SequenceNumberGenerator();
    const msgChunkManager = new source_1.SecureMessageChunkManager("HEL", options, securityHeader, sequenceNumberGenerator);
    const chunks = [];
    let chunkCounter = 0;
    function collect_chunk(chunk) {
        const chunkCopy = Buffer.allocUnsafe(chunk.length);
        chunk.copy(chunkCopy, 0, 0, chunk.length);
        // append the copy to our chunk collection
        chunks.push(chunkCopy);
    }
    msgChunkManager.on("chunk", (chunk, final) => {
        collect_chunk(chunk);
        chunkCounter += 1;
        if (!final) {
            // all packets shall be 'chunkSize'  byte long, except last
            chunk.length.should.equal(options.chunkSize);
        }
        else {
            // last packet is smaller
            // chunk.length.should.equal(  20 +/*padding*/  options.headerSize + options.signatureLength);
            chunkCounter.should.eql(5);
        }
    });
    // feed chunk-manager one byte at a time
    const n = (bodySize) * 4 + 12;
    const buf = Buffer.alloc(1);
    for (let i = 0; i < n; i += 1) {
        buf.writeUInt8(i % 256, 0);
        msgChunkManager.write(buf, 1);
    }
    // write this single buffer
    msgChunkManager.end();
    chunks.length.should.equal(5);
    // checking final flags ...
    chunks.forEach((chunk) => {
        chunk.subarray(0, 3).toString().should.eql("HEL");
    });
    // check lengths
    chunks[0].subarray(4, 8).readUInt32LE(0).should.eql(options.chunkSize);
    chunks[1].subarray(4, 8).readUInt32LE(0).should.eql(options.chunkSize);
    chunks[2].subarray(4, 8).readUInt32LE(0).should.eql(options.chunkSize);
    chunks[3].subarray(4, 8).readUInt32LE(0).should.eql(options.chunkSize);
    chunks[chunks.length - 1].subarray(4, 8).readUInt32LE(0).should.eql(12 + options.signatureLength + headerSize + 8);
    // check final car
    chunks[0].readUInt8(3).should.equal("C".charCodeAt(0));
    chunks[1].readUInt8(3).should.equal("C".charCodeAt(0));
    chunks[2].readUInt8(3).should.equal("C".charCodeAt(0));
    chunks[3].readUInt8(3).should.equal("C".charCodeAt(0));
    chunks[chunks.length - 1].readUInt8(3).should.equal("F".charCodeAt(0));
    if (options.verifyBufferFunc) {
        chunks.forEach(options.verifyBufferFunc);
    }
}
exports.performMessageChunkManagerTest = performMessageChunkManagerTest;
//# sourceMappingURL=signature_helpers.js.map